@@ -2,7 +2,7 @@ from .requestor import APIRequestor
 
 
 def auth_test(error: str = None, **kwargs):
-    """Tests the ability to connect to the API
+    """Tests validity of your credentials
     """
     params = {}
     if error:
