@@ -85,6 +85,23 @@ class Solver:
         buffer.seek(0)
         return buffer
 
+    def get_min_size_from_frac(self, data, outcome, target):
+        if data[outcome].dtype == np.object:
+            return self.min_size_frac * data.loc[data[outcome] == target, outcome].size
+        else:
+            return self.min_size_frac * data[outcome].size
+
+    def set_min_size_frac_from_size(self, data, outcome, target, size):
+        try:
+            if data[outcome].dtype == np.object:
+                self.min_size_frac = min(
+                    0.95, size / data.loc[data[outcome] == target, outcome].size
+                )
+            else:
+                self.min_size_frac = min(0.95, size / data[outcome].size)
+        except ZeroDivisionError:
+            pass
+
     def _run_solve(self, mdf: MondoDataFrame, outcome: str, target: str):
         encoder = DDTransformer(mdf)
         edf = utilities.encode_dataframe(mdf, encoder)
@@ -121,6 +138,14 @@ class Solver:
         rule_encoded = result["rule"]
         rule_decoded = utilities.decode_rule(rule_encoded, encoder)
 
+        discarded_rules = result["diagnostics"].get("discarded_rules", None)
+        discarded_rules_encoded = discarded_rules["rules"]
+        discarded_rules_decoded = [
+            utilities.decode_rule(rule_encoded, encoder)
+            for rule_encoded in discarded_rules_encoded
+        ]
+        discarded_rules["rules"] = discarded_rules_decoded
+
         # Stats
         if mdf[outcome].var_type == "discrete":
             # Discrete Stats - operate in original space
@@ -139,7 +164,7 @@ class Solver:
                 edf, rule_encoded, outcome_encoded, target_encoded
             )
 
-        return rule_decoded, sample_stats, score
+        return rule_decoded, sample_stats, score, discarded_rules
 
     @property
     def max_cycles(self):
@@ -216,7 +241,9 @@ class Solver:
         outcome = m_y.name
         target = m_y.target_class
 
-        self.rule, sample_stats, self.score = self._run_solve(mdf, outcome, target)
+        self.rule, sample_stats, self.score, self.discarded_rules = self._run_solve(
+            mdf, outcome, target
+        )
 
         self.size = sample_stats["size"]
         self.mean = sample_stats["mean"]
@@ -228,7 +255,15 @@ class Solver:
             )
 
     def fit_predicted(
-        self, m_X, m_y, m_y_predicted, predicted=False, max_depth=None, min_score=0,
+        self,
+        m_X,
+        m_y,
+        m_y_predicted,
+        predicted=False,
+        max_depth=None,
+        min_score=0,
+        balance=False,
+        balance_threshold=0.2,
     ):
         """
         Fit the Solver with the provided data but optimizing on the error between
@@ -262,6 +297,21 @@ class Solver:
             Sets a minimum Z-score found by rules as a threshold for rejecting rules.
             If a rule is rejected, exploration down that branch will bet terminated.
 
+        balance: Bool, optional (default=False)
+            Allows the solver to create nested rules of roughly equal size in an
+            effort to balance the overall size of rules. In this process, the
+            Solver.min_size_frac is adjusted to allow subsequent rule size to
+            match the prior rule size as much as possible while maintaining rule
+            stability.
+
+        balance_threshold: float, optional (default=0.2)
+            If balance is set to True, the balance threshold allows setting how much
+            of a relative difference a subsequent rules size should be with respect to
+            the prior rule. This value can be as low as 0 to any multiple desired.
+
+            Example: balance_threshold = 0.2 implies that the subsequent rule size
+            should be within 20% (plus or minus) of the prior rule size
+
         """
 
         explorable_vars = [col for col in m_X.columns if col != m_y.name]
@@ -286,9 +336,17 @@ class Solver:
         outcome = y_error.name
         target = y_error.target_class
 
+        min_size_frac_orig = self.min_size_frac
+
         self.rule_tree = dict()
 
         def evaluate(data, depth=0, current="IN", prior=(0, "IN")):
+            def size_within_threshold(old_size, new_size, threshold=0.2):
+                if old_size == 0:
+                    return False
+                else:
+                    return np.abs(new_size / old_size - 1) <= threshold
+
             if all(
                 (
                     data.shape[0] > 1,
@@ -296,9 +354,13 @@ class Solver:
                     depth <= max_depth if max_depth is not None else depth <= 1e6,
                 )
             ):
-                self.rule, sample_stats, self.score = self._run_solve(
-                    data, outcome, target
-                )
+                min_size_orig = self.get_min_size_from_frac(data, outcome, target)
+                (
+                    self.rule,
+                    sample_stats,
+                    self.score,
+                    self.discarded_rules,
+                ) = self._run_solve(data, outcome, target)
                 self.size = sample_stats["size"]
                 self.mean = sample_stats["mean"]
 
@@ -315,16 +377,53 @@ class Solver:
                     xrule_data = self.get_rule_data(data, xrule=True).copy()
 
                     try:
-                        evaluate(rule_data, depth + 1, "IN", key)
+                        if balance:
+                            if size_within_threshold(
+                                self.size, min_size_orig, balance_threshold
+                            ):
+                                self.set_min_size_frac_from_size(
+                                    rule_data, outcome, target, min_size_orig
+                                )
+                                evaluate(rule_data, depth + 1, "IN", key)
+                            else:
+                                pass
+                        else:
+                            evaluate(rule_data, depth + 1, "IN", key)
                     except NotEnoughPointsError:
                         pass
                     try:
-                        if (key + (depth + 1, "IN")) in self.rule_tree:
+                        if balance:
+                            if size_within_threshold(
+                                self.size, min_size_orig, balance_threshold
+                            ):
+                                self.set_min_size_frac_from_size(
+                                    xrule_data, outcome, target, min_size_orig
+                                )
+                                evaluate(xrule_data, depth + 1, "OUT", key)
+                            else:
+                                pass
+                        else:
                             evaluate(xrule_data, depth + 1, "OUT", key)
                     except NotEnoughPointsError:
                         pass
 
         evaluate(mdf)
+        self.min_size_frac = min_size_frac_orig
+
+    def write_rule_tree(self, file_handle):
+        """
+        Export rule tree to json format.
+
+        Parameters
+        ----------
+        file_handle: string
+            Location of file to write the tree to.
+
+        """
+        if self.rule_tree:
+            utilities.write_rule_tree(self.rule_tree, file_handle)
+        else:
+            raise ValueError("Rule Tree not yet set")
 
     def fit_meta(self, m_X, m_y, exp: BaseExploration):
         """
@@ -420,10 +519,11 @@ class Solver:
         )
         y_train.target_class = y_test.target_class = m_y.target_class
 
+        full_mdf = MondoDataFrame(pd.concat((MondoDataFrame(m_y), m_X), axis=1))
         mdf = MondoDataFrame(pd.concat((MondoDataFrame(y_test), X_test), axis=1))
 
         # For use in scoring later (_run_solve handles in-flight encoding)
-        encoder = DDTransformer(mdf)
+        encoder = DDTransformer(full_mdf)
         edf = utilities.encode_dataframe(mdf, encoder)
 
         train_scores, train_sizes = [], []
@@ -434,16 +534,19 @@ class Solver:
             X_train_curr = X_train.iloc[train_index]
             y_train_curr = y_train.iloc[train_index]
 
-            mdf_cur = MondoDataFrame(
+            mdf_curr = MondoDataFrame(
                 pd.concat((MondoDataFrame(y_train_curr), X_train_curr), axis=1)
             )
 
             outcome = m_y.name
             target = m_y.target_class
 
-            train_rule, train_sample_stats, train_score = self._run_solve(
-                mdf_cur, outcome, target
-            )
+            (
+                train_rule,
+                train_sample_stats,
+                train_score,
+                discarded_rules,
+            ) = self._run_solve(mdf_curr, outcome, target)
             train_size = train_sample_stats["size"]
 
             if m_y.var_type == "discrete":
@@ -455,16 +558,16 @@ class Solver:
                 test_size = test_sample_stats["size"]
             else:
                 # Continous Stats - operate in encoded space
-                test_rule = utilities.encode_rule(train_rule, encoder)
+                rule_encoded = utilities.encode_rule(train_rule, encoder)
                 outcome_encoded = encoder._column_encoder[outcome]
 
-                edf_applied = utilities.apply_rule_to_df(edf, test_rule)
+                edf_applied = utilities.apply_rule_to_df(edf, rule_encoded)
 
                 test_sample_stats = utilities.get_stats(
                     edf_applied, outcome_encoded, target
                 )
                 test_score = utilities.score_rule(
-                    edf, test_rule, outcome_encoded, target
+                    edf, rule_encoded, outcome_encoded, target
                 )
                 test_size = test_sample_stats["size"]
 
